#!/usr/bin/perl

# Generate instance of initial and terminal state with state generator.
# For use with ToH.lp

# Written by Alex Westlund and Shaden Smith under Mirek Truszczynski
# Fall 2010

if( scalar(@ARGV) != 4 )
{
	die "usage: $0 numDisks initialState terminalState instanceNum\nWhere instanceNum corresponds to the output filename.";
}

$pegs = 4;
$disks = $ARGV[0];
$initialSkipped = $ARGV[1];
$terminal = $ARGV[2];
$pNum = $ARGV[3];

$solLength = $terminal - $initialSkipped;

if( $disks < 4 )
{
	die "4 disk minimum. Sorry!\n";
}

$STATEGENERATOR = "./stateGenerator.py";

@state = `$STATEGENERATOR $pegs $disks 0`;

foreach (@state)
{
	# We must ensure that the solution length does not exceed the total length
	# of the problem
	if( $_ =~ /Moves: (\d+)/ )
	{
		$totalLength = $1;
	}
}

if( $terminal > $totalLength )
{
	die "Solution length is greater than total problem length. Must be less than or equal to $totalLength moves.\n";
}

# Calc maxInt for filename
if( $totalLength > $numDisks )
{
	$maxInt = $totalLength;
}
else
{
	$maxInt = $numDisks;
}

# open output file
$outFile = "$pNum-hanoi-$maxInt-0.asp";
open(OUTPUT, ">$outFile") or die $!;

# Grab initial and terminal states for output
@initialState = `$STATEGENERATOR $pegs $disks $initialSkipped`;
$termState = $terminal;
@terminalState = `$STATEGENERATOR $pegs $disks $termState`;

# print ASCII images of initial and terminal states
print OUTPUT "% Initial state:\n";
foreach (@initialState)
{
	if( $_ =~ /[A-Z]:/ )
	{
		print OUTPUT "% " . $_;
	}
}
print OUTPUT "\n% Terminal state:\n";
foreach (@terminalState)
{
	if( $_ =~ /[A-Z]:/ )
	{
		print OUTPUT "% " . $_;
	}
}

print OUTPUT "\n% $initialSkipped initial moves skipped.\n";
print OUTPUT "% $solLength move solution.\n";

print OUTPUT "steps($solLength).\n";
for($i = 0; $i <= $solLength; $i++)
{
	print OUTPUT "time($i).\n";
}

# print disks
$outdisks = $disks + $pegs;
for($i = 1; $i <= $outdisks; $i++)
{
	print OUTPUT "disk($i).\n";
}

$currPeg = 1;

# Grab state information
foreach (@initialState)
{
	if( $_ =~ /[A-Z]: (.+)/ )
	{
		@pegData = split(/ /, $1);
		# Print bottom disk
		$bottom = $pegData[0] + $pegs; # +$pegs because pegs are disks
		print OUTPUT "on0($bottom,$currPeg).\n";
		# Each proceeding disk
		for( $i = 1; $i < scalar(@pegData); $i++ )
		{
			$top = $pegData[$i]+$pegs;		# curr
			$bottom = $pegData[$i-1]+$pegs;	# prev
			print OUTPUT "on0($top,$bottom).\n";
		}
	}
	if( $_ =~ /[A-Z]:/ )
	{
		$currPeg++;
	}
}

$currPeg = 1;
print OUTPUT "\n\n% Terminal State: \n";
foreach (@terminalState)
{
	if( $_ =~ /[A-Z]: (.+)/ )
	{
		@pegData = split(/ /, $1);
		# Print bottom disk
		$bottom = $pegData[0] + $pegs; # +$pegs because pegs are disks
		print OUTPUT "ongoal($bottom,$currPeg).\n";
		# Each proceeding disk
		for( $i = 1; $i < scalar(@pegData); $i++ )
		{
			$top = $pegData[$i]+$pegs;		# curr
			$bottom = $pegData[$i-1]+$pegs;	# prev
			print OUTPUT "ongoal($top,$bottom).\n";
		}
	}
	if( $_ =~ /[A-Z]:/ )
	{
		$currPeg++;
	}
}

close OUTPUT;

